
This package implements a new module nn.DAG which inherits from nn.Container and allows to combine modules in an arbitrary graph without cycle.

#Example#

A typical use would be:

```Lua
model = nn.DAG()

a = nn.Linear(100, 10)
b = nn.ReLU()
c = nn.Linear(10, 15)
d = nn.CMulTable()
e = nn.Linear(15, 15)

model:addEdge(a, b)
model:addEdge(b, nn.Linear(10, 15), nn.ReLU(), d)
model:addEdge(b, c)
model:addEdge(c, d)
model:addEdge(c, nn.Mul(-1), e)

model:setInput(a)
model:setOutput({ d, e })

input = torch.Tensor(30, 100):uniform()
output = model:updateOutput(input)

```

which would encode the following graph

                 +- Linear(10, 10) -> ReLU ---> d -->
                /                              /
               /                              /
    --> a --> b -----------> c --------------+
                              \
                               \
                                +-- Mul(-1) --> e -->

and run a forward pass with a random batch of 30 samples.

Note that DAG:addEdge allows to add a bunch of edges at once. This is particularly useful to add anonymous modules which have a single predecessor and successor.

#Input and output#

If a node has a single successor, its output is sent unchanged as input to that successor. If it has multiple successors, the outputs are collected into a table, and the table is used as input to the successor node. The indexes of the outputs in that table reflects the order in which they appear in the addEdge commands.

The expected input (respectively the produced output) is a nested table of inputs reflecting the structure of the nested table of modules provided to DAG:setInput (respectively DAG:setOutput)

So for instance, in the example above, the model expects a tensor as input, since it is the input to the module a, and its output will is a table composed of two tensors, corresponding to the outputs of d and e respectively.

*Francois Fleuret, Jan 12th, 2017*
